--- src/w32.h.original	2008-01-10 21:16:16.000000000 +0900
+++ src/w32.h	2008-04-24 00:44:33.769063800 +0900
@@ -129,6 +129,7 @@
 
 extern void init_ntproc (void);
 extern void term_ntproc (void);
+extern void syms_of_w32 (void);
 extern void globals_of_w32 (void);
 extern void syms_of_w32term (void);
 extern void syms_of_w32fns (void);
--- src/w32.c.original	2008-02-23 22:49:09.000000000 +0900
+++ src/w32.c	2008-04-24 00:42:22.761991200 +0900
@@ -75,6 +75,11 @@
 #include <windows.h>
 #include <shlobj.h>
 
+#include <shellapi.h>
+#ifndef FOF_NO_CONNECTED_ELEMENTS
+#define FOF_NO_CONNECTED_ELEMENTS 0x2000 /* don't operate on connected elements. */
+#endif /* FOF_NO_CONNECTED_ELEMENTS */
+
 #ifdef HAVE_SOCKETS	/* TCP connection support, if kernel can do it */
 #include <sys/socket.h>
 #undef socket
@@ -104,6 +109,7 @@
 typedef HRESULT (WINAPI * ShGetFolderPath_fn)
   (IN HWND, IN int, IN HANDLE, IN DWORD, OUT char *);
 
+void syms_of_w32 ();
 void globals_of_w32 ();
 
 extern Lisp_Object Vw32_downcase_file_names;
@@ -111,6 +117,8 @@
 extern Lisp_Object Vw32_get_true_file_attributes;
 extern int w32_num_mouse_buttons;
 
+int w32_delete_to_recycle_bin;
+
 
 /*
   Initialization states
@@ -2243,17 +2251,58 @@
 int
 sys_rmdir (const char * path)
 {
-  return _rmdir (map_w32_filename (path, NULL));
+  if (w32_delete_to_recycle_bin)
+    return (sys_unlink (path));
+  else
+    return _rmdir (map_w32_filename (path, NULL));
 }
 
 int
 sys_unlink (const char * path)
 {
-  path = map_w32_filename (path, NULL);
+  if (w32_delete_to_recycle_bin)
+    {
+      SHFILEOPSTRUCT file_op;
+      /* `pFrom' member of struct SHFILEOPSTRUCT:
+	 Each file name must be terminated by a single NULL
+	 character. An additional NULL character must be appended to the
+	 end of the final name to indicate the end of pFrom. */
+      char tmp_path[MAX_PATH + 1];
+      Lisp_Object tmp;
+
+      /* Must use only absolute file names with SHFileOperation(). */
+      tmp = Fexpand_file_name (build_string (path), Qnil);
+
+      path = map_w32_filename (SDATA (tmp), NULL);
+
+      /* On Unix, unlink works without write permission. */
+      _chmod (path, 0666);
+
+      memset (tmp_path, 0, sizeof (tmp_path));
+      strcpy(tmp_path, path);
+
+      memset(&file_op, 0, sizeof (file_op));
+      file_op.hwnd = HWND_DESKTOP;
+      file_op.wFunc = FO_DELETE;
+      file_op.pFrom = tmp_path;
+      file_op.pTo = NULL;
+      file_op.fFlags = FOF_SILENT | FOF_NOCONFIRMATION | FOF_ALLOWUNDO
+	| FOF_NOERRORUI | FOF_NO_CONNECTED_ELEMENTS;
+      file_op.fAnyOperationsAborted = FALSE;
+      file_op.hNameMappings = NULL;
+      file_op.lpszProgressTitle = NULL;
+
+      /* value returned by SHFileOperation() must match value returned by _unlink() */
+      return (SHFileOperation(&file_op) == 0 ? 0 : -1);
+    }
+  else
+    {
+      path = map_w32_filename (path, NULL);
 
-  /* On Unix, unlink works without write permission. */
-  _chmod (path, 0666);
-  return _unlink (path);
+      /* On Unix, unlink works without write permission. */
+      _chmod (path, 0666);
+      return _unlink (path);
+    }
 }
 
 static FILETIME utc_base_ft;
@@ -4160,6 +4209,16 @@
   SetConsoleCtrlHandler(shutdown_handler, TRUE);
 }
 
+void
+syms_of_w32 ()
+{
+  DEFVAR_BOOL ("w32-delete-to-recycle-bin",
+	       &w32_delete_to_recycle_bin,
+	       doc: /* Non-nil means delete file or directory to recycle bin.
+Default is nil, which means just delete it.  */);
+  w32_delete_to_recycle_bin = 0;
+}
+
 /* end of w32.c */
 
 /* arch-tag: 90442dd3-37be-482b-b272-ac752e3049f1
--- src/emacs.c.original	2008-01-10 21:16:14.000000000 +0900
+++ src/emacs.c	2008-04-24 00:43:32.060947200 +0900
@@ -1588,6 +1588,7 @@
       syms_of_vmsproc ();
 #endif /* VMS */
 #ifdef WINDOWSNT
+      syms_of_w32 ();
       syms_of_ntproc ();
 #endif /* WINDOWSNT */
       syms_of_window ();
